@@ -1,7 +1,7 @@
 # encoding: utf8
 """
 Implementation of the "low-level" functionality used by the common
-implementation of the API.
+implementation of the API, for the NEURON simulator.
 
 Functions and classes useable by the common implementation:
 
@@ -23,17 +23,20 @@ Attributes:
 
 All other functions and classes are private, and should not be used by other
 modules.
-    
-$Id: simulator.py 702 2010-02-03 10:39:32Z apdavison $
+
+:copyright: Copyright 2006-2011 by the PyNN team, see AUTHORS.
+:license: CeCILL, see LICENSE for details.
+
+$Id: simulator.py 957 2011-05-03 13:44:15Z apdavison $
 """
 
 from pyNN import __path__ as pyNN_path
-from pyNN import common, random
+from pyNN import common, errors, core
 import platform
 import logging
 import numpy
 import os.path
-from neuron import h
+from neuron import h, load_mechanisms
 
 # Global variables
 nrn_dll_loaded = []
@@ -44,20 +47,6 @@ logger = logging.getLogger("PyNN")
 
 # --- Internal NEURON functionality -------------------------------------------- 
 
-def load_mechanisms(path=pyNN_path[0]):
-    # this now exists in the NEURON distribution, so could probably be removed
-    global nrn_dll_loaded
-    if path not in nrn_dll_loaded:
-        arch_list = [platform.machine(), 'i686', 'x86_64', 'powerpc']
-        # in case NEURON is assuming a different architecture to Python, we try multiple possibilities
-        for arch in arch_list:
-            lib_path = os.path.join(path, 'hoc', arch, '.libs', 'libnrnmech.so')
-            if os.path.exists(lib_path):
-                h.nrn_load_dll(lib_path)
-                nrn_dll_loaded.append(path)
-                return
-        raise Exception("NEURON mechanisms not found in %s." % os.path.join(path, 'hoc'))
-
 def is_point_process(obj):
     """Determine whether a particular object is a NEURON point process."""
     return hasattr(obj, 'loc')
@@ -93,7 +82,7 @@ def nativeRNG_pick(n, rng, distribution='uniform', parameters=[0,1]):
 def h_property(name):
     """Return a property that accesses a global variable in Hoc."""
     def _get(self):
-        return getattr(h,name)
+        return getattr(h, name)
     def _set(self, val):
         setattr(h, name, val)
     return property(fget=_get, fset=_set)
@@ -116,7 +105,7 @@ class _Initializer(object):
         """
         h('objref initializer')
         h.initializer = self
-        self.fih = h.FInitializeHandler(0, "initializer._initialize()")
+        self.fih = h.FInitializeHandler(1, "initializer._initialize()")
         self.clear()
     
     def register(self, *items):
@@ -125,7 +114,7 @@ class _Initializer(object):
         objects must have a `memb_init()` method.
         """
         for item in items:
-            if isinstance(item, common.Population):
+            if isinstance(item, (common.BasePopulation, common.Assembly)):
                 if "Source" not in item.celltype.__class__.__name__: # don't do memb_init() on spike sources
                     self.population_list.append(item)
             else:
@@ -158,12 +147,13 @@ class _State(object):
         h('tstop = 0')
         h('steps_per_ms = 1/dt')
         self.parallel_context = h.ParallelContext()
-        self.parallel_context.spike_compress(1,0)
+        self.parallel_context.spike_compress(1, 0)
         self.num_processes = int(self.parallel_context.nhost())
         self.mpi_rank = int(self.parallel_context.id())
         self.cvode = h.CVode()
         h('objref plastic_connections')
         self.clear()
+        self.default_maxstep=10.0
     
     t = h_property('t')
     def __get_dt(self):
@@ -189,20 +179,22 @@ def reset():
     state.running = False
     state.t = 0
     state.tstop = 0
+    h.finitialize()
 
 def run(simtime):
     """Advance the simulation for a certain time."""
     if not state.running:
         state.running = True
-        local_minimum_delay = state.parallel_context.set_maxstep(10)
+        local_minimum_delay = state.parallel_context.set_maxstep(state.default_maxstep)
         h.finitialize()
         state.tstop = 0
+        logger.debug("default_maxstep on host #%d = %g" % (state.mpi_rank, state.default_maxstep ))
         logger.debug("local_minimum_delay on host #%d = %g" % (state.mpi_rank, local_minimum_delay))
         if state.num_processes > 1:
-            assert local_minimum_delay >= state.min_delay,\
+            assert local_minimum_delay >= state.min_delay, \
                    "There are connections with delays (%g) shorter than the minimum delay (%g)" % (local_minimum_delay, state.min_delay)
     state.tstop += simtime
-    logger.info("Running the simulation for %d ms" % simtime)
+    logger.info("Running the simulation for %g ms" % simtime)
     state.parallel_context.psolve(state.tstop)
 
 def finalize(quit=False):
@@ -237,7 +229,7 @@ class ID(int, common.IDMixin):
         """
         gid = int(self)
         self._cell = cell_model(**cell_parameters)          # create the cell object
-        register_gid(gid, self._cell.source, section=self._cell) # not adequate for multi-compartmental cells
+        register_gid(gid, self._cell.source, section=self._cell.source_section)
         if hasattr(self._cell, "get_threshold"):            # this is not adequate, since the threshold may be changed after cell creation
             state.parallel_context.threshold(int(self), self._cell.get_threshold()) # the problem is that self._cell does not know its own gid
         
@@ -253,59 +245,15 @@ class ID(int, common.IDMixin):
         for name, val in parameters.items():
             setattr(self._cell, name, val)
         
-
-# --- For implementation of create() and Population.__init__() -----------------
-
-def create_cells(cellclass, cellparams, n, parent=None):
-    """
-    Create cells in NEURON.
-    
-    `cellclass`  -- a PyNN standard cell or a native NEURON cell class that
-                   implements an as-yet-undescribed interface.
-    `cellparams` -- a dictionary of cell parameters.
-    `n`          -- the number of cells to create
-    `parent`     -- the parent Population, or None if the cells don't belong to
-                    a Population.
-    
-    This function is used by both `create()` and `Population.__init__()`
-    
-    Return:
-        - a 1D array of all cell IDs
-        - a 1D boolean array indicating which IDs are present on the local MPI
-          node
-        - the ID of the first cell created
-        - the ID of the last cell created
-    """
-    assert n > 0, 'n must be a positive integer'
-    if isinstance(cellclass, basestring): # cell defined in hoc template
-        try:
-            cell_model = getattr(h, cellclass)
-        except AttributeError:
-            raise common.InvalidModelError("There is no hoc template called %s" % cellclass)
-        cell_parameters = cellparams or {}
-    elif isinstance(cellclass, type) and issubclass(cellclass, common.StandardCellType):
-        celltype = cellclass(cellparams)
-        cell_model = celltype.model
-        cell_parameters = celltype.parameters
-    else:
-        cell_model = cellclass
-        cell_parameters = cellparams
-    first_id = state.gid_counter
-    last_id = state.gid_counter + n - 1
-    all_ids = numpy.array([id for id in range(first_id, last_id+1)], ID)
-    # mask_local is used to extract those elements from arrays that apply to the cells on the current node
-    mask_local = all_ids%state.num_processes==state.mpi_rank # round-robin distribution of cells between nodes
-    for i,(id,is_local) in enumerate(zip(all_ids, mask_local)):
-        all_ids[i] = ID(id)
-        all_ids[i].parent = parent
-        if is_local:
-            all_ids[i].local = True
-            all_ids[i]._build_cell(cell_model, cell_parameters)
-        else:
-            all_ids[i].local = False
-    initializer.register(*all_ids[mask_local])
-    state.gid_counter += n
-    return all_ids, mask_local, first_id, last_id
+    def get_initial_value(self, variable):
+        """Get the initial value of a state variable of the cell."""
+        return getattr(self._cell, "%s_init" % variable)
+        
+    def set_initial_value(self, variable, value):
+        """Set the initial value of a state variable of the cell."""
+        index = self.parent.id_to_local_index(self)
+        self.parent.initial_values[variable][index] = value
+        setattr(self._cell, "%s_init" % variable, value)
 
 
 # --- For implementation of connect() and Connector classes --------------------
@@ -352,7 +300,8 @@ class Connection(object):
         self.pre2wa.delay = self.nc.delay * (1-ddf)
         self.pre2wa.weight[0] = 1
         # directly create NetCon as wa is on the same machine as the post-synaptic cell
-        self.post2wa = h.NetCon(self.target._cell.source, self.weight_adjuster)
+        self.post2wa = h.NetCon(self.target._cell.source, self.weight_adjuster,
+                                sec=self.target._cell.source_section)
         self.post2wa.threshold = 1
         self.post2wa.delay = self.nc.delay * ddf
         self.post2wa.weight[0] = -1
@@ -395,7 +344,7 @@ def generate_synapse_property(name):
         if hasattr(synapse, name):
             setattr(synapse, name, val)
         else:
-            raise Exception("synapse type does not have an attribute 'U'")
+            raise Exception("synapse type does not have an attribute '%s'" % name)
     return property(_get, _set)
 setattr(Connection, 'U', generate_synapse_property('U'))
 setattr(Connection, 'tau_rec', generate_synapse_property('tau_rec'))
@@ -428,9 +377,10 @@ class ConnectionManager(object):
         Create a new ConnectionManager.
         
         `synapse_model` -- either None or 'Tsodyks-Markram'.
-        `parent` -- the parent `Projection`, if any.
+        `parent` -- the parent `Projection`
         """
         global connection_managers
+        assert parent is not None
         self.connections = []
         self.parent = parent
         self.synapse_type = synapse_type
@@ -439,7 +389,16 @@ class ConnectionManager(object):
         
     def __getitem__(self, i):
         """Return the `i`th connection on the local MPI node."""
-        return self.connections[i]
+        if isinstance(i, int):
+            if i < len(self):
+                return self.connections[i]
+            else:
+                raise IndexError("%d > %d" % (i, len(self)-1))
+        elif isinstance(i, slice):
+            if i.stop < len(self):
+                return [self.connections[j] for j in range(*i.indices(i.stop))]
+            else:
+                raise IndexError("%d > %d" % (i.stop, len(self)-1))
     
     def __len__(self):
         """Return the number of connections on the local MPI node."""
@@ -449,6 +408,12 @@ class ConnectionManager(object):
         """Return an iterator over all connections on the local MPI node."""
         return iter(self.connections)
     
+    def _resolve_synapse_type(self):
+        if self.synapse_type is None:
+            self.synapse_type = weight>=0 and 'excitatory' or 'inhibitory'
+        if self.synapse_model == 'Tsodyks-Markram' and 'TM' not in self.synapse_type:
+            self.synapse_type += '_TM'  
+    
     def connect(self, source, targets, weights, delays):
         """
         Connect a neuron to one or more other neurons with a static connection.
@@ -462,8 +427,8 @@ class ConnectionManager(object):
         """
         if not isinstance(source, int) or source > state.gid_counter or source < 0:
             errmsg = "Invalid source ID: %s (gid_counter=%d)" % (source, state.gid_counter)
-            raise common.ConnectionError(errmsg)
-        if not common.is_listlike(targets):
+            raise errors.ConnectionError(errmsg)
+        if not core.is_listlike(targets):
             targets = [targets]
         if isinstance(weights, float):
             weights = [weights]
@@ -472,23 +437,69 @@ class ConnectionManager(object):
         assert len(targets) > 0
         for target in targets:
             if not isinstance(target, common.IDMixin):
-                raise common.ConnectionError("Invalid target ID: %s" % target)
+                raise errors.ConnectionError("Invalid target ID: %s" % target)
               
-        assert len(targets) == len(weights) == len(delays), "%s %s %s" % (len(targets),len(weights),len(delays))
+        assert len(targets) == len(weights) == len(delays), "%s %s %s" % (len(targets), len(weights), len(delays))
+        self._resolve_synapse_type()
         for target, weight, delay in zip(targets, weights, delays):
             if target.local:
+                if "." in self.synapse_type: 
+                    section, synapse_type = self.synapse_type.split(".") 
+                    synapse_object = getattr(getattr(target._cell, section), synapse_type) 
+                else: 
+                    synapse_object = getattr(target._cell, self.synapse_type) 
+                nc = state.parallel_context.gid_connect(int(source), synapse_object)
+                nc.weight[0] = weight
+                
+                # if we have a mechanism (e.g. from 9ML that includes multiple
+                # synaptic channels, need to set nc.weight[1] here
+                if nc.wcnt() > 1:
+                    nc.weight[1] = target._cell.type.synapse_types.index(self.synapse_type)
+                nc.delay  = delay
+                # nc.threshold is supposed to be set by ParallelContext.threshold, called in _build_cell(), above, but this hasn't been tested
+                self.connections.append(Connection(source, target, nc))
+
+    def convergent_connect(self, sources, target, weights, delays):
+        """
+        Connect a neuron to one or more other neurons with a static connection.
+        
+        `sources`  -- a list/1D array of pre-synaptic cell IDs, or a single ID.
+        `target` -- the ID of the post-synaptic cell.
+        `weight`  -- a list/1D array of connection weights, or a single weight.
+                     Must have the same length as `targets`.
+        `delays`  -- a list/1D array of connection delays, or a single delay.
+                     Must have the same length as `targets`.
+        """
+        if not isinstance(target, int) or target > state.gid_counter or target < 0:
+            errmsg = "Invalid target ID: %s (gid_counter=%d)" % (target, state.gid_counter)
+            raise errors.ConnectionError(errmsg)
+        if not core.is_listlike(sources):
+            sources = [sources]
+        if isinstance(weights, float):
+            weights = [weights]
+        if isinstance(delays, float):
+            delays = [delays]
+        assert len(sources) > 0
+        for source in sources:
+            if not isinstance(source, common.IDMixin):
+                raise errors.ConnectionError("Invalid source ID: %s" % source)
+              
+        assert len(sources) == len(weights) == len(delays), "%s %s %s" % (len(sources),len(weights),len(delays))
+                
+        if target.local:
+            for source, weight, delay in zip(sources, weights, delays):
                 if self.synapse_type is None:
-                    self.synapse_type = weight>=0 and 'excitatory' or 'inhibitory'
+                    self.synapse_type = weight >= 0 and 'excitatory' or 'inhibitory'
                 if self.synapse_model == 'Tsodyks-Markram' and 'TM' not in self.synapse_type:
                     self.synapse_type += '_TM'
-                synapse_object = getattr(target._cell, self.synapse_type)
+                synapse_object = getattr(target._cell, self.synapse_type)  
                 nc = state.parallel_context.gid_connect(int(source), synapse_object)
                 nc.weight[0] = weight
                 nc.delay  = delay
                 # nc.threshold is supposed to be set by ParallelContext.threshold, called in _build_cell(), above, but this hasn't been tested
                 self.connections.append(Connection(source, target, nc))
 
-    def get(self, parameter_name, format, offset=(0,0)):
+    def get(self, parameter_name, format):
         """
         Get the values of a given attribute (weight, delay, etc) for all
         connections on the local MPI node.
@@ -496,8 +507,6 @@ class ConnectionManager(object):
         `parameter_name` -- name of the attribute whose values are wanted.
         `format` -- "list" or "array". Array format implicitly assumes that all
                     connections belong to a single Projection.
-        `offset` -- an (i,j) tuple giving the offset to be used in converting
-                    source and target IDs to array indices.
         
         Return a list or a 2D Numpy array. The array element X_ij contains the
         attribute value for the connection from the ith neuron in the pre-
@@ -513,7 +522,7 @@ class ConnectionManager(object):
             values = numpy.nan * numpy.ones((self.parent.pre.size, self.parent.post.size))
             for c in self.connections:
                 value = getattr(c, parameter_name)
-                addr = (c.source-offset[0], c.target-offset[1])
+                addr = (self.parent.pre.id_to_index(c.source), self.parent.post.id_to_index(c.target))
                 if numpy.isnan(values[addr]):
                     values[addr] = value
                 else:
@@ -532,13 +541,12 @@ class ConnectionManager(object):
                    or a 2D array with the same dimensions as the connectivity
                    matrix (as returned by `get(format='array')`).
         """
-        if common.is_number(value):
+        if numpy.isscalar(value):
             for c in self:
                 setattr(c, name, value)
         elif isinstance(value, numpy.ndarray) and len(value.shape) == 2:
-            offset = (self.parent.pre.first_id, self.parent.post.first_id)
             for c in self.connections:
-                addr = (c.source-offset[0], c.target-offset[1])
+                addr = (self.parent.pre.id_to_index(c.source), self.parent.post.id_to_index(c.target))
                 try:
                     val = value[addr]
                 except IndexError, e:
@@ -547,7 +555,7 @@ class ConnectionManager(object):
                     raise Exception("Array contains no value for synapse from %d to %d" % (c.source, c.target))
                 else:
                     setattr(c, name, val)
-        elif common.is_listlike(value):
+        elif core.is_listlike(value):
             for c,val in zip(self.connections, value):
                 setattr(c, name, val)
         else:
@@ -556,8 +564,9 @@ class ConnectionManager(object):
 
 # --- Initialization, and module attributes ------------------------------------
 
-load_mechanisms() # maintains a list of mechanisms that have already been imported
+mech_path = os.path.join(pyNN_path[0], 'neuron', 'nmodl')
+load_mechanisms(mech_path) # maintains a list of mechanisms that have already been imported
 state = _State()  # a Singleton, so only a single instance ever exists
 del _State
 initializer = _Initializer()
-del _Initializer
\ No newline at end of file
+del _Initializer
